function isNumber(x) {
  return parseFloat(x) == x
};

function isMobile() {
    return getComputedStyle(document.querySelector('.special-column')).paddingRight == '0px'
}

function isInView(el) {
    var rect = el.getBoundingClientRect();
    var elemTop = rect.top;
    var elemBottom = rect.bottom;

    // Only completely visible elements return true:
    var isVisible = (elemTop >= 0) && (elemBottom <= window.innerHeight);
    // Partially visible elements return true:
    //isVisible = elemTop < window.innerHeight && elemBottom >= 0;
    return isVisible;
}

function parseGalleryHash() {
    let parts = document.location.hash.slice(1).split('/')
    if (parts.lengh == 1 && isNumber(parts[0])) {
        return {
            'gallery': 'inline',
            "image": parseInt(parts[0], 10) - 1,
        }
    } else if (parts.length  == 2 && isNumber(parts[0]) && isNumber(parts[1])) {
        return {
            "gallery": parseInt(parts[0], 10) - 1,
            "image": parseInt(parts[1], 10) - 1,
        }
    }
    return {}
}

function loadGalleries() {
    let current = parseGalleryHash()
    let galleryIdx = 0
    document.querySelectorAll('.gallery').forEach(gallery => {
        var photos = [], dialog
        gallery.querySelectorAll(".photo").forEach(img => {
            const src = img.src.replace('_thumbnail', '_display')
            let photo = {
                src: src,
                caption: img.dataset.caption,
                orig: img.dataset.orig,
                edit: img.dataset.edit
            }
            photos.push(photo)
            img.parentElement.href = `#${galleryIdx+1}/${photos.length}`
            img.parentElement.addEventListener("click", event => {
                event.preventDefault()
                event.stopPropagation()
                dialog.open(src, -1, event)
            })
        })
        if (isMobile()) {
            dialog = loadInlineGallery(photos, galleryIdx, null, gallery)
        } else {
            dialog = loadGallery(photos, galleryIdx, null, gallery)
        }
        dialog.element && gallery.appendChild(dialog.element)
        if (current.gallery == galleryIdx) {
            dialog.open(undefined, current.image)
        }
        galleryIdx += 1
    })
    document.querySelectorAll('.inline-gallery').forEach(gallery => {
        var photos = [], stage
        gallery.querySelectorAll(".photo").forEach(img => {
            const src = img.src.replace('_thumbnail', '_display')
            let photo = {
                src: src,
                caption: img.dataset.caption,
                orig: img.dataset.orig
            }
            photos.push(photo)
            img.parentElement.addEventListener("click", event => {
                event.preventDefault()
                event.stopPropagation()
                dialog.open(src, -1, event)
            })
        })
        const main = document.querySelector('.gallery-main-stage')
        stage = loadGallery(photos, 'inline', main, gallery)
    })
}

function loadInlineGallery(images, idx, main, thumbnails) {
    thumbnails.querySelectorAll('.photo').forEach(img => {
        const container = document.createElement('div')
        container.classList.add("inline-photo")
        container.innerHTML = `
            <img class="photo" src="${img.src}">
            <div class="caption">${img.dataset.caption||''}</div>
        `
        img.replaceWith(container)
    })

    function scroll(event) {
        let found = false
        let index = 0
        thumbnails.querySelectorAll('.photo').forEach(img => {
            if (!found) {
                if (isInView(img)) {
                    found = true
                    const hash = `#${idx + 1}/${index + 1}`
                    if (document.location.hash != hash) {
                        document.location.hash = hash
                    }
                }
            }
            index += 1
        })
        /*
        // fixme this jumps don't do it
        if (!found && document.location.hash.slice(1).length) {
            document.location.hash = ''
        }
        */
    }

    return {
        element: null,
        open: function(src, newIdx=-1, event=null) {
            let img
            if (newIdx != -1) {
                img = thumbnails.querySelectorAll('.photo')[newIdx - 1]
            }
            if (thumbnails.classList.contains("inline")) {
                thumbnails.classList.remove("inline");
                thumbnails.querySelectorAll('.photo').forEach(img => {
                    img.src = img.src.replace('_display', '_thumbnail')
                })
                thumbnails.previousElementSibling.scrollIntoView()
                window.removeEventListener("scroll", scroll)
            } else {
                thumbnails.classList.add("inline");
                thumbnails.querySelectorAll('.photo').forEach(img => {
                    img.src = img.src.replace('_thumbnail', '_display')
                })
                setTimeout(() => {
                    if (img) {
                        img.scrollIntoView()
                    } else if (event && event.target) {
                        event.target.scrollIntoView()
                    }
                }, 20)
                window.addEventListener("scroll", scroll)
            }
        }
    }
}
function loadGallery(images, idx, main, thumbnails) {
    let gallery;

    if (idx == 'inline') {
        gallery = main
    } else {
        gallery = document.createElement("dialog")
        gallery.classList.add("gallery-dialog")
        gallery.innerHTML = `
            <div class="photo-container">
                <div class="image">
                    <img class="main-image" src="" alt="">
                </div>
                <button class="prev nav-btn">❮ Prev</button>
                <button class="next nav-btn">Next ❯</button>
            </div>
            <button class="close">&#10006;</button>
            <a class="edit disabled" target="_blank" title="edit">&#9998;</a>
            <a class="download disabled" target="_blank" download title="download original">
                <svg xmlns="http://www.w3.org/2000/svg" width="14" height="14" viewBox="0 0 304 384"><path fill="currentColor" d="M299 128L149 277L0 128h85V0h128v128h86zM0 320h299v43H0v-43z"/></svg>
            </a>
            <div class="caption"></div>
            <div class="thumbnails"></div>
        `

        gallery.addEventListener("click", event => {
            if (event.target.className == "close") {
                document.location.hash = ''
                gallery.close();
                return
            }
            var rect = gallery.getBoundingClientRect();
            var isInDialog = (rect.top <= event.clientY && event.clientY <= rect.top + rect.height &&
                rect.left <= event.clientX && event.clientX <= rect.left + rect.width);
            if (!isInDialog) {
                document.location.hash = ''
                gallery.close();
            }
        })
    }
    let currentIndex = 0;

    const mainImage = gallery.querySelector(".main-image");
    const caption = gallery.querySelector(".caption");
    const prevBtn = gallery.querySelector(".prev.nav-btn");
    const nextBtn = gallery.querySelector(".next.nav-btn");
    const download = gallery.querySelector(".download");
    const edit = gallery.querySelector(".edit");
    let thumbnailsContainer
    if (idx != 'inline') {
        thumbnailsContainer = gallery.querySelector(".thumbnails");
    }

    gallery.addEventListener("keydown", event => {
        if (event.keyCode == 39) {
            event.preventDefault()
            event.stopPropagation()
            nextBtn.onclick(event)
        } else if (event.keyCode == 37) {
            event.preventDefault()
            event.stopPropagation()
            prevBtn.onclick(event)
        }
    })

    function updateGallery(index) {
        mainImage.src = images[index].src.replace('_display', '_thumbnail');
        setTimeout(() => {
            mainImage.src = images[index].src;
        }, 0)
        caption.innerHTML = images[index].caption;
        if (idx == 'inline') {
            thumbnails.querySelectorAll(".thumbnail").forEach((thumb, i) => {
                thumb.classList.toggle("active", i === index);
            });
        } else {
            gallery.querySelectorAll(".thumbnail").forEach((thumb, i) => {
                thumb.classList.toggle("active", i === index);
            });
            if (thumbnailsContainer && !isMobile()) {
                gallery.querySelector('.active').scrollIntoView()
            }
        }
        if (images[index].orig) {
            download.href = images[index].orig
            download.classList.remove('disabled')
        } else {
            download.href = ""
            download.classList.add('disabled')
        }
        if (edit) {
            if (images[index].edit) {
                edit.href = images[index].edit
                edit.classList.remove('disabled')
            } else {
                edit.href = ""
                edit.classList.add('disabled')
            }
        }
        prevBtn.disabled = index === 0;
        nextBtn.disabled = index === images.length - 1;
        if (idx == 'inline') {
            document.location.hash = `#${index + 1}`
        } else {
            document.location.hash = `#${idx + 1}/${index + 1}`
        }
    }

    function createThumbnails() {
        images.forEach((img, index) => {
            const thumb = document.createElement("img");
            thumb.src = img.src.replace('_display', '_thumbnail');
            thumb.classList.add("thumbnail");
            thumb.onclick = (event) => {
                event.preventDefault()
                event.stopPropagation()
                currentIndex = index;
                updateGallery(index);
            };
            thumbnailsContainer.appendChild(thumb);
        });
    }

    function showGallery() {
        window.dialog = gallery
        gallery.showModal()
    }

    prevBtn.onclick = () => {
        event.preventDefault()
        event.stopPropagation()
        if (currentIndex > 0) {
            currentIndex--;
            updateGallery(currentIndex);
        }
    };

    nextBtn.onclick = () => {
        event.preventDefault()
        event.stopPropagation()
        if (currentIndex < images.length - 1) {
            currentIndex++;
            updateGallery(currentIndex);
        }
    };

    idx != 'inline' && createThumbnails();
    updateGallery(currentIndex);

    const imagesIndex = images.map(img => img.src);
    return {
        element: gallery,
        open: function(src, newIdx=-1) {
            if (newIdx != -1) {
                currentIndex = newIdx
            } else {
                currentIndex = imagesIndex.indexOf(src)
            }
            updateGallery(currentIndex)
            if (idx != 'inline') {
                showGallery()
            }
        }
    }
}
window.addEventListener("load", loadGalleries)
